#include <stdio.h>
#include <stdlib.h>
#ifdef BSD
# include <strings.h>
#else
# include <string.h>
#endif
#include <lhash.h>
#include <mcs.h>

#include "mcs_cache.h"

static LHASH *mcs_hash=(LHASH*)0;

typedef struct _mcs_hash_entry {
	char *name;
	int status;
	mcs_table_t *table;
} mcs_hash_entry_t;

static unsigned long mcs_hash_cb(mcs_hash_entry_t *hash_entry) {
	return lh_strhash(hash_entry->name);
}

static int mcs_cmp_cb(mcs_hash_entry_t *e1,mcs_hash_entry_t *e2) {
	return strcmp(e1->name,e2->name);
}

/* this function shall only be invoked under mutex lock.
   name and table must be newly malloc()ed objects! */
int mcs_newtable(char *name,mcs_table_t *table,int status) {
	mcs_hash_entry_t *new_entry;
	if (!mcs_hash) mcs_hash=lh_new(mcs_hash_cb,mcs_cmp_cb);
	if (!mcs_hash) return 1;
	new_entry=(mcs_hash_entry_t*)malloc(sizeof(mcs_hash_entry_t));
	if (!new_entry) {
		lh_free(mcs_hash);
		return 1;
	}
	new_entry->name=name;
	new_entry->status=status;
	new_entry->table=table;
	/* lh_insert probably may run into no memory condition
	   but eay would not tell us about it... So, pretend it
	   always works good. */
	lh_insert(mcs_hash,(char*)new_entry);
	return 0;
}

/* look up table in the hash.  Return NULL if not found. */
mcs_table_t *mcs_findtable(char *name,int *statusp) {
	mcs_hash_entry_t dummy_entry,*ret_entry;

	if (statusp) *statusp=1;
	if (!mcs_hash) return (mcs_table_t*)0;
	dummy_entry.name=name;
	ret_entry=(mcs_hash_entry_t*)lh_retrieve(mcs_hash,(char*)&dummy_entry);
	if (ret_entry) {
		if (statusp) *statusp=ret_entry->status;
		return ret_entry->table;
	}
	else return (mcs_table_t*)0;
}
